//**********************************************************************************************************\
//
//	environment.c:	Smoothing out platform and compiler variations. 
//
//**********************************************************************************************************/


#include "project.h"

//



///////////////////////////////////////////////////////////////////////////////
//	Simple memory
///////////////////////////////////////////////////////////////////////////////

void* Memory_free (void* ptr)
{
	if (ptr == NULL)		Host_FatalError ("Memory_free failed: \"%s\"", "Attempted to free NULL pointer");
	
	free (ptr);  ptr = NULL;

	return ptr;  // We set ptr to NULL (our local copy) and return ptr for consistency.
}


void* Memory_malloc (const size_t size, const char* Reason)
{
	void* ptr				= malloc (size);
	
	if (ptr == NULL)		Host_FatalError ("Memory_malloc failed: \"%s\" on %i bytes", Reason, size);
	return ptr;
}

void* Memory_calloc (const size_t n, const size_t size, const char* Reason)
{
	void* ptr				= calloc (n, size);
	const int allocsize		= (int)(n * size);
	
	if (ptr == NULL)		Host_FatalError ("Memory_calloc: \"%s\" on %i bytes", Reason, allocsize);
	return ptr;
}

//
// strlcpy and strlcat
//



//========================================================
// strlcat and strlcpy, from OpenBSD

/*
 * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef HAVE_STRLCAT
size_t strlcat(char *dst, const char *src, size_t siz)
{
	register char *d = dst;
	register const char *s = src;
	register size_t n = siz;
	size_t dlen;
	
	/* Find the end of dst and adjust bytes left but don't go past end */
	while (n-- != 0 && *d != '\0')
		d++;
	dlen = d - dst;
	n = siz - dlen;
	
	if (n == 0)
		return(dlen + strlen(s));
	while (*s != '\0')
	{
		if (n != 1)
		{
			*d++ = *s;
			n--;
		}
		s++;
	}
	*d = '\0';
	
	return(dlen + (s - src));	/* count does not include NUL */
}
#endif  // #ifndef HAVE_STRLCAT


#ifndef HAVE_STRLCPY

size_t strlcpy(char *dst, const char *src, size_t siz)
{
	char *d = dst;
	const char *s = src;
	size_t n = siz;
	
	/* Copy as many bytes as will fit */
	if (n != 0 && --n != 0)
	{
		do
		{
			if ((*d++ = *s++) == 0)
				break;
		} while (--n != 0);
	}
	
	/* Not enough room in dst, add NUL and traverse rest of src */
	if (n == 0)
	{
		if (siz != 0)
			*d = '\0';		/* NUL-terminate dst */
		while (*s++)
			;
	}
	
	return(s - src - 1);	/* count does not include NUL */
}


#endif  // #ifndef HAVE_STRLCPY


#ifdef __BIG_ENDIAN__

long    LittleLong (long myLong)
{
	byte_1 = (myLong >>  0) & 255;
	byte_2 = (myLong >>  8) & 255;
	byte_3 = (myLong >> 16) & 255;
	byte_4 = (myLong >> 24) & 255;

	return ((long) byte_1 << 24) + ((long) byte_2 << 16) + ((long) byte_3 << 8 ) + byte_4;
}

#endif

#ifdef _WIN32
int System_GetCurrentWorkingDirectory (char* buffer, fbool PrintIt)
{
	// Get the current working directory: 
	if ( _getcwd( buffer, MAX_PATH_IS_1024 ) == NULL )
	{
      perror( "_getcwd error" );
	  printf ("Unable to get current directory\n");
	  return 0;
	}

	String_Edit_SlashesForward_Like_Unix (buffer);
	
	if (PrintIt) printf ("Current working directory: %s\n", buffer);
	return 1;
}
#endif


/*
=================
CheckParm

Checks for the given parameter in the program's command line arguments
Returns the argument number (1 to argc-1) or 0 if not present
=================
*/
// set these before calling CheckParm
int commandLineArgumentsCount;
char **commandLineArguments;


int CheckParm (char *check)
{
	int             i;

	for (i = 1; i < commandLineArgumentsCount; i++)
	{
		if ( StringMatchCaseless(check, commandLineArguments[i]) )
			return i;
	}

	return 0;
}



void printDateAndTime (void)
{
// http://www.cplusplus.com/forum/windows/18617/
// http://www.cplusplus.com/reference/clibrary/ctime/localtime/
  time_t rawtime;
  struct tm * timeinfo;

  time ( &rawtime );
  timeinfo = localtime ( &rawtime );
  printf ( "Current local time and date: %s\n", asctime (timeinfo) );
}  

void Enter_To_Exit_DebugOnly (void)
{
#ifndef _DEBUG	// Always do the pause thing in debug mode
	if (!CheckParm ("-pause"))
#endif
	{
		printf ("\nPress enter to continue");
		getchar();
	}
}



/////////////////////////
// MOVE THESE

int RoundUp_To_Power_Of_Two (const int myInteger)
{
	int Upsized_POW2;
	for (Upsized_POW2 = 1 ; Upsized_POW2 < myInteger ; Upsized_POW2 <<= 1) //  <<= 1 is a bitwise double, same as "times two"
		;

	return Upsized_POW2;
}

float Center_Position_BigSize_LittleSizef (float biglength, float littlelength)
{
	return biglength  / 2 - littlelength  / 2;
}
#define roundf(x)   (floorf((x) + 0.5f)) // If we don't have C99 ... this is a bit wrong but whatever
int Center_Position_BigSize_LittleSize_Int (int biglength, int littlelength)
{
	return (int)roundf( Center_Position_BigSize_LittleSizef((float)biglength, (float)littlelength) ); // Subtract 1 from each before function?  Or after?
}

fbool ScreenRect_DoOverlap (const ScreenRect myRect, const ScreenRect otherRect)
{	
	const fbool xOverlap = myRect.topLeft.x <= otherRect.bottomRight.x && otherRect.topLeft.x <= myRect.bottomRight.x;
	const fbool yOverlap = myRect.topLeft.y <= otherRect.bottomRight.y && otherRect.topLeft.y <= myRect.bottomRight.y;
	
	return xOverlap && yOverlap;	
}