//**********************************************************************************************************\
//
//	files.c:	Doing file work
//
//**********************************************************************************************************/

#include "project.h"

#define isFolder(x) (!!(x.attrib & _A_SUBDIR ))


// Pak file structure
// 1. Header
// 2. File bytes one after another
// 3. Directory

//typedef struct
//{
//	char	name[56];
//	int		filepos, filelen;
//} packfile_t;
//
//typedef struct
//{
//	char	id[4];
//	int		dirofs;
//	int		dirlen;
//} packheader_t;



int File_Length_ByHandle (FILE* fileToCheck)
{
	int lengthOfFile;

	fseek (fileToCheck, 0, SEEK_END);	// Go to end
	lengthOfFile = ftell(fileToCheck);	// Read position, that is file length

	fseek (fileToCheck, 0, SEEK_SET);	// Set to start
	return lengthOfFile;
}

fbool File_Write_Binary (const char* fileToWrite, byte* data, int numBytes)
{
	FILE*	fout	= fopen (fileToWrite, "wb");

	if (!fout)											return False;
	if ( fwrite (data, numBytes, 1, fout) != 1)			return False;

	fclose (fout);

	return True;
}




int File_Append_BinaryFile (FILE* writeHandle, const char* fileToRead)
{
	int		bytesWritten = 0;
	FILE*	readHandle = fopen (fileToRead, "rb");
	
	if (readHandle)
	{
		char	buf[4096];
		int		bufsize = sizeof(buf);

		int fileReadLength	= File_Length_ByHandle (readHandle);
		int remaining		= fileReadLength;

		printf ("Appending %i bytes from %s\n", fileReadLength, fileToRead);
	
		//CreatePath (dest);
	
		while (remaining)
		{
			int bytesThisPass = remaining < bufsize ? remaining : bufsize;

			File_Safe_Read  (readHandle,  buf, bytesThisPass);   // Read 4096 bytes in (or less if near EOF)
			File_Safe_Write (writeHandle, buf, bytesThisPass);  // Write 4096 bytes out 

			remaining    -= bytesThisPass;
			bytesWritten += bytesThisPass;
		}

		fclose (readHandle);
	} 

	return bytesWritten;
}

int File_GetCursor (FILE* fileHandle)
{
	int position = ftell (fileHandle);

	return position;
}

void File_SetCursor_ToStart (FILE* fileHandle)
{
	fseek  (fileHandle, 0, SEEK_SET);
}

int File_Safe_Read (FILE *f, void *buffer, size_t count)
{
	int bytesRead = fread (buffer, 1, count, f);
	
	if (bytesRead == count)
		return bytesRead;

//	Error_Exit ("File read failure");
	return False;
}


int File_Safe_Write (FILE *f, void *buffer, size_t count)
{
	int bytesWritten = fwrite (buffer, 1, count, f);

	if ( bytesWritten == count)
		return bytesWritten;

//	Error_Exit ("File write failure");
	return False;
}


int _FileMatch_Count (const char* dirString, const char* filter, fbool doFolders)
{
	char	queryString[MAX_PATH_IS_1024];
	int		count = 0;

	SNPrintf (queryString, sizeof(queryString), "%s/%s", dirString, filter);
	
	do
	{
		struct	_finddata_t fileinfo;
		int					handle = _findfirst (queryString, &fileinfo);
		int				    foundFile = 1;
		
		if (handle == -1)
			break; // No matches

		for (  ; foundFile != -1; foundFile = _findnext( handle, &fileinfo ))
			if ( isFolder (fileinfo) == doFolders )
				if (doFolders == False /* unconditionally*/ || (StringNOTMatch (fileinfo.name, ".") && StringNOTMatch (fileinfo.name, "..")) /* no special dirs*/)  
					count ++;
		
		_findclose (handle);

	} while (0);

	return count;
}




void _FileMatch_Populate (const char* dirString, const char* filter, text1024_t* myFiles, int countMyFiles, fbool doFolders)
{
	char	queryString[MAX_PATH_IS_1024];
	int		count = 0;

	SNPrintf (queryString, sizeof(queryString), "%s/%s", dirString, filter);
	
	do
	{
		struct	_finddata_t fileinfo;
		int					handle = _findfirst (queryString, &fileinfo);
		int				    foundFile = 1;
		
		if (handle == -1)
			break; // No matches

		for (  ; foundFile != -1 && count < countMyFiles; foundFile = _findnext( handle, &fileinfo ))
			if ( isFolder (fileinfo) == doFolders )
				if (doFolders == False /* unconditionally*/ || (StringNOTMatch (fileinfo.name, ".") && StringNOTMatch (fileinfo.name, "..")) /* no special dirs*/)  
					SNPrintf (myFiles[count++].text, 1024, "%s/%s", dirString, fileinfo.name);
		
		_findclose (handle);

	} while (0);

	if (count > countMyFiles) 
		printf ("Warning: Count of %i exceeded expected count of %i", count, countMyFiles);

//	return count;

}

void sFileMatch_Populate (const char* dirString, const char* filter, text1024_t* myFiles, int countMyFiles)
{
	_FileMatch_Populate (dirString, filter, myFiles, countMyFiles, False /* do files */ );
}

void sFolderMatch_Populate (const char* dirString, const char* filter, text1024_t* myFiles, int countMyFiles)
{
	_FileMatch_Populate (dirString, filter, myFiles, countMyFiles, True /* do folders */ );
}

int sFileMatch_Count (const char* dirString, const char* filter)
{
	return _FileMatch_Count (dirString, filter, False /* don't do folders */);	
}


int sFolderMatch_Count (const char* dirString, const char* filter)
{
	return _FileMatch_Count (dirString, filter, True /* do folders */);	
}

text1024_t* FileMatch_Populate_Alloc (const char* dirString, const char* filter, int* count)
{
	// 1. Find out number of matches.  2. Allocate space if there are matches otherwise set fileSpace to NULL
	int			matches		= sFileMatch_Count (dirString, filter);
	text1024_t* fileSpace	= matches ? Memory_calloc (matches, sizeof(text1024_t), "filename space") : NULL;

	if (matches) 
		sFileMatch_Populate (dirString, filter, fileSpace, matches);

	// Set count and return memory address
	*count = matches;
	return fileSpace;
}

text1024_t* FolderMatch_Populate_Alloc (const char* dirString, const char* filter, int* count)
{
	// 1. Find out number of matches.  2. Allocate space if there are matches otherwise set folderSpace to NULL
	int			matches		= sFolderMatch_Count (dirString, filter);
	text1024_t* folderSpace	= matches ? Memory_calloc (matches, sizeof(text1024_t), "filename space") : NULL;

	if (matches) 
		sFolderMatch_Populate (dirString, filter, folderSpace, matches);

	// Set count and return memory address
	*count = matches;
	return folderSpace;
}
