//**********************************************************************************************************\
//
//	image.h:	Image functions
//
//**********************************************************************************************************/


#ifndef _IMAGE_FUNCS_H
#define _IMAGE_FUNCS_H



// Rarely needed constants, make them self explain
#define RGBA_BytesPerPixel_Is_4 4
#define RGBA_BitsPerPixel_Is_32	32

typedef enum
{
// alpha info ... these are bit flags but in this case we can combine	
	COLORINFO_ALPHA_SOLID_WHITE = 0,
	COLORINFO_ALPHA_ALPHA_MASK	= 1,
	COLORINFO_ALPHA_CHANNEL		= 2,
	COLORINFO_ALPHA_SOLID_BLACK = 3,
	COLORINFO_ALPHA_INFO		= 3,  // Mask value to this to read above
} colorinfo;


typedef enum
{
	PIXOP_NEGATIVE_IMAGE,
	PIXOP_GREYSCALE,
	PIXOP_SWAP_RED_GREEN,
	PIXOP_SWAP_GREEN_BLUE,
	PIXOP_SWAP_BLUE_RED,
	PIXOP_ALPHA_FROM_COLOR,
	PIXOP_REMOVE_RED,
	PIXOP_REMOVE_GREEN,
	PIXOP_REMOVE_BLUE,
	PIXOP_QUAKE_COLORS_NO_FULLBRIGHT,
	
} pixelop;

typedef enum
{
	TRANSFORM_FLIP_VERTICAL,
	TRANSFORM_FLIP_HORIZONTAL,
	TRANSFORM_ROTATE_CLOCKWISE90,
	TRANSFORM_ROTATE_COUNTERCLOCKWISE90,	
	TRANSFORM_ROTATE_180,
} transform_simple_op;


typedef enum
{
	PASTE_REPLACE,
// Alpha operations?
} pasteop;

typedef struct
{
	size_t		maxPossibleColors;		// Table maximum size
	unsigned*	colors;
	
	int			numColors;			// Actual colors found
} palette_t;

int Palette_AlphaInfo_From_GreyInfo (fbool foundGrey, fbool foundWhite, fbool foundBlack);

void PixelComponent_Swap (byte* component1, byte* component2);
byte PixelComponent_Get_Average (byte* red, byte* green, byte *blue);



///////////////////////////////////////////////////////////////////////////////
// Pixel RGBA creation FROM (look at data and build pixel)
///////////////////////////////////////////////////////////////////////////////


unsigned Pixel_RGBA_From_Components (byte red, byte green, byte blue, byte alpha);
unsigned Pixel_RGBA_From_Components_Percents (float redPercent, float greenPercent, float bluePercent, float alphaPercent);
unsigned Pixel_RGBA_PixelOperation (unsigned myPixel, pixelop pixelOperation);
unsigned Pixel_RGBA_PasteOp_Blend (pasteop pasteOp, unsigned sourcePixel, unsigned destPixel);
unsigned Pixel_RGBA_PixelOperation (unsigned myPixel, pixelop pixelOperation);


///////////////////////////////////////////////////////////////////////////////
// Pixel RGBA out TO creation (Look at pixel and make some other data)
///////////////////////////////////////////////////////////////////////////////

void Pixel_RGBA_To_Components (unsigned pixel, byte* red, byte* green, byte *blue, byte* alpha);
void Pixel_RGBA_To_Components_Percents (unsigned pixel, float* redPercent, float* greenPercent, float* bluePercent, float* alphaPercent);


///////////////////////////////////////////////////////////////////////////////
// Pixel RGBA out TO creation where this is general information
///////////////////////////////////////////////////////////////////////////////


void Pixel_RGBA_To_AlphaInfo (unsigned pixel, fbool* foundGrey, fbool* foundWhite, fbool* foundBlack);
void Pixel_RGBA_CheckAdd_To_Palette (unsigned pixel, palette_t* palette);


typedef struct
{
	byte*			data;
	size_t			byteCount;		// width height x 4
	int				pixelcount;
	int				width;
	int				height;
	
} imageRGBA_t;

// Rawest form:  No special data types (enums are just ints).


int Image_RGBA_GetInfo_AlphaType		(const byte *buffer, size_t buffersize, int pixelCount, int width, int height);
int Image_RGBA_GetInfo_NumColorsUsed	(const byte *source, size_t sourceSize, int sourcePixelCount, int sourceWidth, int sourceHeight);

byte* Image_RGBA_Modify_RectArea (pixelop pixelOperation, int x1, int y1, int x2, int y2, byte *buffer, size_t bufferSize, int pixelCount, int width, int height);
byte* Image_RGBA_Modify (pixelop pixelOperation, byte *buffer, size_t bufferSize, int pixelCount, int width, int height);

byte* Image_RGBA_PasteAt (pasteop pasteOp, int x, int y, const byte* source, size_t sourceSize, int sourcePixelCount, int sourceWidth, int sourceHeight, byte* dest, size_t destSize, int destPixelCount, int destWidth, int destHeight);
byte* Image_RGBA_Transform (transform_simple_op transformOperation, byte *buffer, size_t bufferSize, int pixelCount, int* width, int* height);


//void ResampleTexture (const unsigned *indata, const int inwidth, const int inheight, unsigned *outdata, const int outwidth, const int outheight, const fbool quality);



// Image Loading
byte* Image_RGBA_TGA_Read_OpenFile_Alloc (FILE *fin, const char* filename_nopath, int* my_image_width, int* my_image_height, fbool* loadError);
fbool Image_RGBA_TGA_Write_ToOpenFile (FILE *fout, const char* filename_nopath, const byte *pixels, int width, int height, int BitsPerPixel, fbool IsUpsideDown, const byte* header, unsigned char headerSize);
fbool Image_RGBA_TGA_Write_File (const char* filename, const byte* imageData, int width, int height, const byte* header, unsigned char headerSize);


// More refined form: More "user-friendly where user is the coder form"

int    Image_Info_AlphaType			(const imageRGBA_t* myImage);
int    Image_Info_NumColorsUsed		(const imageRGBA_t* myImage);
fbool  Image_Info_IsSizePower2		(const imageRGBA_t* myImage);
fbool  Image_Info_NewSizeFitsImage	(const imageRGBA_t* myImage, int width, int height);
fbool  Image_Info_NewSizeIsSame		(const imageRGBA_t* myImage, int width, int height);
void   Image_Info_CenterRectOfSize	(const imageRGBA_t* myImage, int width, int height, int* rectCenteredAtX, int* rectCenteredAtY);

// Pre-Import
fbool  Image_Query_File_Dimensions	(const char* fileName, int* setWidth, int* setHeight);

// Import
fbool  Image_Import_FileX			(imageRGBA_t* myImage, const char* loadFileName);
fbool  Image_Import_TGA_OpenFile	(imageRGBA_t* myImage, FILE* tgaFileHandle);

// Export

fbool  Image_Export_Lmp_File		(const imageRGBA_t* myImage, const char* fileToWrite);
fbool  Image_Export_TGA_File		(const imageRGBA_t* myImage, const char* fileToWrite);
fbool  Image_Export_TGA_File_With_Header (const imageRGBA_t* myImage, const char* fileToWrite, const byte* header, unsigned char headerSize);

fbool  Image_Export_Mip_File		(const imageRGBA_t* myImage, const char* fileName, const char* textureName);
byte*  Image_Export_QPAL_Data		(const imageRGBA_t* myImage);


fbool Image_Export_Mip_OpenFile		(const imageRGBA_t* myImage, FILE* fout, const char* textureName);

// Image creation and retiring
void   Image_Create_UsingData		(imageRGBA_t* myImage, int width, int height, byte* pData);			// Fills in image data from existing image
void   Image_Create_New_Alloc		(imageRGBA_t* myImage, int width, int height, unsigned backgroundColor);	// Allocates a new buffer of width/height and fills in data
void   Image_Create_Duplicate_Alloc	(imageRGBA_t* myImage, imageRGBA_t* duplicateImage);


void   Image_Destroy_KeepStats_free (imageRGBA_t* myImage);
void   Image_Destroy				(imageRGBA_t* myImage);

// Enlargement
void   Image_EnlargeTo_Realloc		(imageRGBA_t* myImage, int newWidth, int newHeight, unsigned backgroundColor);
void   Image_EnlargeTo_POW2_Realloc	(imageRGBA_t* myImage);

// Modification
void   Image_Do_PixelOperation		(imageRGBA_t* myImage, const pixelop myPixelOperation);
void   Image_Do_PasteOperationAt	(imageRGBA_t* myImage, pasteop pasteOp, imageRGBA_t* pasteSource, int pasteAtX, int pasteAtY);





// Internal?
void  Image_Replace_free			(imageRGBA_t* exitingImage, imageRGBA_t* receivingImage);





// Needed for color manipulation
void Pixel_RGBA_To_Components_Percents (unsigned pixel, float* redPercent, float* greenPercent, float* bluePercent, float* alphaPercent);
void Pixel_RGBA_To_Components (unsigned pixel, byte* red, byte* green, byte *blue, byte* alpha);

byte PixelByte_Get_Best_Quake_NoFullbright (byte red, byte green, byte blue);


unsigned Pixel_RGBA_From_Components_Percents (float redPercent, float greenPercent, float bluePercent, float alphaPercent);
unsigned Pixel_RGBA_From_Components (byte red, byte green, byte blue, byte alpha);


#endif // _IMAGE_FUNCS_H
