//**********************************************************************************************************\
//
//	project.h:	Main project header
//
//**********************************************************************************************************/

// Headers
#include <direct.h>
#include <stdlib.h>
#include <stdio.h>
#include <tchar.h>
#include <stddef.h> 
#include <string.h>
#include <math.h>
#include <time.h>
#include <io.h>

#include "environment.h"
#include "image.h"

// Mathlike stuffs?

#define SQUARED(x)	((x)*(x))
#define IS_MULTIPLE_OF_16(number) (fmod (number, 16) == 0)

typedef struct				{ int x;   int y;							} ScreenXY;
typedef struct				{ ScreenXY topLeft;   ScreenXY bottomRight;	} ScreenRect;

#define SCREENRECT_OF_OVERLAP(rc1, rc2) {                                                          \
{   FLOOR(rc1.topLeft.x,     rc2.topLeft.x),       FLOOR(rc1.topLeft.y,     rc2.topLeft.y)    },   \
{ CEILING(rc1.bottomRight.x, rc2.bottomRight.x), CEILING(rc1.bottomRight.y, rc2.bottomRight.y)} }


fbool ScreenRect_DoOverlap (const ScreenRect myRect, const ScreenRect otherRect);

int System_GetCurrentWorkingDirectory (char* buffer, fbool PrintIt);

// File system
#define MAX_PATH_IS_1024					1024
#define MAX_WAD_TEXTURE_NAME_LENGTH_IS_15	15
#define MAX_PAK_FILENAME_IS_55				55

typedef struct
{
	char	text[1024];

} text1024_t;



// String functions
#define StringMatch(s1, s2)						!strcmp(s1, s2)
#define StringMatchCaseless(s1, s2)				!strcasecmp(s1, s2)
#define StringMatchNCaseless(s1, s2, s3)		!strncasecmp(s1, s2, s3)

#define StringNOTMatch(s1, s2)					(strcmp(s1, s2) !=0 )
#define StringNOTMatchCaseless(s1, s2)			(strcasecmp(s1, s2) !=0 )

#define StringStartsWith(s1, s2)				!strncmp(s1, s2, strlen(s2))
#define StringStartsWithCaseLess(s1, s2)		!strncasecmp(s1, s2, strlen(s2))

#define StringLCopy(dest, source)				strlcpy(dest, source, sizeof(dest))
#define StringLCat(dest, stringToAppend)		strlcat(dest, stringToAppend, sizeof(dest))
#define StringInStr(bigString, findWhat)		(strstr(bigString, findWhat) != NULL)

void String_Edit_SlashesForward_Like_Unix (char *WindowsStylePath);
void String_Edit_StripExtension (char* myEditString);
char* String_Edit_Append_To_Name (char* filename, const char* appendToName);

int File_HasDotExtension (const char *filename, const char *extension);
char* String_SkipPath (const char *pathName);

text1024_t* FolderMatch_Populate_Alloc (const char* dirString, const char* filter, int* count);
text1024_t* FileMatch_Populate_Alloc (const char* dirString, const char* filter, int* count);

int   File_Append_BinaryFile	(FILE* writeHandle, const char* fileToRead);
int   File_GetCursor			(FILE* fileHandle);
void  File_SetCursor_ToStart	(FILE* fileHandle);
fbool File_Write_Binary			(const char* writeFileName, byte* data, int numBytes);
//fbool File_Write_LMP (const char* fileToWrite, byte* LMP_Pixels, long width, long height, int numBytes);


int File_Safe_Read (FILE *f, void *buffer, size_t count);


int File_Safe_Write (FILE *f, void *buffer, size_t count);

int PakFile_Create_From_List (char* outputPakName, text1024_t* listOfFiles, int numFiles);


struct pak_access_s*  PakFile_Build_Start     (const char* outPakFileName);
fbool				  PakFile_Build_Add_File  (struct pak_access_s* myPack, const char* newFileName);
struct pak_access_s*  PakFile_Build_Finish    (struct pak_access_s* myPack);

struct wad_access_s*  WAD2File_Build_Start    (const char* outWAD2FileName);
struct wad_access_s*  WAD3File_Build_Start    (const char* outWAD2FileName);
fbool				  WADFile_Build_Add_File (struct wad_access_s* myWAD2, const char* newFileName);
//fbool				  WAD2File_Build_Add_Data (struct wad_access_s* myWAD2, const byte* data, int count);
struct wad_access_s*  WADFile_Build_Finish   (struct wad_access_s* myWAD2);
//  Homelesses ...

int RoundUp_To_Power_Of_Two (const int myInteger);
int Center_Position_BigSize_LittleSize_Int (int biglength, int littlelength);